//@author: vvvv group
//@help: basic pixel based lightning with point light
//@tags: shading, blinn
//@credits:

struct VertexInputType //VS_IN
{
    float4 Position 	: POSITION;
    float4 UV 			: TEXCOORD0;
	float4 Normal 		: NORMAL;
};

struct PixelInputType // vs2ps
{
    float4 HPosition	: SV_POSITION;
   	float4 UVha			: TEXCOORD0; // dot prods against half-angle
    float4 UVna			: TEXCOORD1; // dot-prods against normal
    float4 UV			: TEXCOORD2; // plain-ol' UV
    float4 diffCol		: COLOR0;
    float4 specCol		: COLOR1;
	float4 UVha2			: TEXCOORD3; // dot prods against half-angle
    float4 UVna2			: TEXCOORD4; // dot-prods against normal
    
    float4 diffCol2		: COLOR2;
    float4 specCol2		: COLOR3;
};

Texture2D halfAngleMap <string uiname="halfAngleMap";>;
Texture2D normalAngleMap <string uiname="normalAngleMap";>;
Texture2D colorTexture <string uiname="colorTexture";>;

SamplerState linearSampler <string uiname="Sampler1 State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

TextureCube cubeMap <string uiname="cubeMap";>;

SamplerState cubeMapSampler <string uiname="Sampler2 State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tV: VIEW;
	float4x4 tP: PROJECTION;
	float4x4 tVP: VIEWPROJECTION;
}; 

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4x4 tWIT: WORLDINVERSETRANSPOSE;
	float4x4 tWVP : WORLDVIEWPROJECTION;
	float4x4 tVI: VIEWINVERSE;
	
	float3 LightPos <float uimin=-100.0; float uimax=100.0;> = { 0.00, 5.00, -2.00 };
	
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 LightColor <bool color=true;String uiname="LightColor";> = { 0.8f,0.8f,1.0f,1.0f };
	float4 AmbiColor <bool color=true;String uiname="AmbiColor";> = { 0.1f,0.1f,0.1f,1.0f };
	float4 SurfColor <bool color=true;String uiname="SurfColor";> = { 1.0f,1.0f,1.0f,1.0f };
	
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};

	//Hilight Strength
	float Ks <float uimin=0.0; float uimax=1.0;> = 0.5;
	//Fresnel Strength
	float Kr <float uimin=0.0; float uimax=1.0;> = 0.6;
	//Fresnel Expon
	float FresExp <float uimin=1.0; float uimax=10.0;> = 5.0;

// Vertex Shader Fresnel
////////////////////////////////////////////////////////////////////////////////

PixelInputType Fresnel_VS(VertexInputType input)
{
    PixelInputType output;

    float3 worldNormal = mul(input.Normal, tWIT).xyz;
	
    worldNormal = normalize(worldNormal);
	
    float4 Po = float4(input.Position.xyz,1);
	
    //compute worldspace position
    float3 worldSpacePos = mul(Po, tW).xyz;
	
    float3 EyePos = tVI[3].xyz;
	
    float3 vertToEye = normalize(worldSpacePos - EyePos);
	
    float fake = abs(dot(vertToEye,worldNormal));
	
    fake = Kr * (1.0 - pow(fake,FresExp));
	
    output.UVha.xyz = reflect(vertToEye,worldNormal);
    output.UVha.w = 1.0;
    output.UVna = input.UV;
    output.UV = input.UV;
    output.diffCol = SurfColor * AmbiColor;
    output.specCol = float4(fake.xxx,1.0);
    output.HPosition = mul(Po, tWVP);
	
    return output;
}

// Pixel Shader Fresnel
////////////////////////////////////////////////////////////////////////////////

float4 Fresnel_PS(PixelInputType input) : SV_TARGET

{
    float4 env = cubeMap.Sample(cubeMapSampler, input.UVha.xyz);
    return (input.diffCol * colorTexture.Sample(linearSampler,input.UVna.xy) + input.specCol * env);
}

// Vertex Shader Cook-Torrance
////////////////////////////////////////////////////////////////////////////////

PixelInputType CookTorrance_VS(VertexInputType input)
{
    PixelInputType output;

	float3 worldNormal = mul(input.Normal, tWIT).xyz;
    
	worldNormal = normalize(worldNormal);

    //build float4
	float4 Po = float4(input.Position.xyz,1.0);
	
    //compute worldspace position
    float3 worldSpacePos = mul(Po, tW).xyz;
	
    float3 LightVec = normalize(LightPos - worldSpacePos);
	
    float ldn = dot(LightVec,worldNormal);
	
    float diffComp = max(0,ldn);
	
    float4 diffContrib = SurfColor * diffComp * LightColor;

    output.diffCol = diffContrib;
    output.diffCol.w = 1.0;
    output.UV = input.UV;

    float3 EyePos = tVI[3].xyz;
	
    float3 vertToEye = normalize(EyePos - worldSpacePos);
	
    float3 halfAngle = normalize(vertToEye + LightVec);
	
    float4 halfIndices = float4(0.5+dot(LightVec, halfAngle)/2.0,
				1.0 - (0.5+dot(worldNormal, halfAngle)/2.0),0.0,1.0);
	
    float4 normIndices = float4(0.5+dot(LightVec, worldNormal)/2.0,
			   1.0 - (0.5+dot(worldNormal, vertToEye)/2.0),0.0,1.0);
	   
	
    float3 vertToEye2 = normalize(worldSpacePos - EyePos);
	
    float fake = abs(dot(vertToEye2,worldNormal));
    output.UVha = halfIndices;
    output.UVna = normIndices;
	
    output.specCol = float4(0,0,0,1);

    // transform into homogeneous-clip space
    output.HPosition = mul(Po, tWVP);
	
	output.UVha2.xyz = reflect(vertToEye2,worldNormal);
    output.UVha2.w = 1.0;
    output.UVna2 = input.UV;
   
    output.diffCol2 = SurfColor * AmbiColor;
    output.specCol2 = float4(fake.xxx,1.0);
    
	
    return output;
}

// Pixel Shader Cook-Torrance
////////////////////////////////////////////////////////////////////////////////

float4 CookTorrance_PS(PixelInputType input) : SV_TARGET

{
    float4 nspec = halfAngleMap.Sample(linearSampler, input.UVha.xy) * 
				   normalAngleMap.Sample(linearSampler, input.UVna.xy) * Ks * LightColor;
	float AlphaZ = 0.1;
    float4 ndiff = colorTexture.Sample(linearSampler, input.UV.xy) * input.diffCol;
	float4 env = cubeMap.Sample(cubeMapSampler, input.UVha.xyz);
//    env = (input.diffCol2 * colorTexture.Sample(linearSampler,input.UVna2.xy) + input.specCol2 * env);
	
	return env ;//(ndiff + nspec +env)/2;
}

// Depth-Stencil State
////////////////////////////////////////////////////////////////////////////////

DepthStencilState Depth_Fresnel
{
	DepthEnable = TRUE;
	DepthWriteMask = ALL;
	DepthFunc = LESS_EQUAL;
	StencilEnable = TRUE;
};

DepthStencilState Depth_CookTorrance
{
	DepthEnable = TRUE;
};

// Blend State
////////////////////////////////////////////////////////////////////////////////

BlendState Blend_Fresnel
{
};

BlendState Blend_CookTorrance
{
	AlphaToCoverageEnable = TRUE;
	BlendEnable[0] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = ADD;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0F;
};

// Rasterizer State
////////////////////////////////////////////////////////////////////////////////

RasterizerState Rasterizer_Fresnel
{
	CullMode = NONE;
};

RasterizerState Rasterizer_CookTorrance
{
	CullMode = NONE;
};

////////////////////////////////////////////////////////////////////////////////

technique10 CookTorranceFresnel <
	string Script = "Pass=p0;Pass=p1";
> {
//    pass p0 <
//	string Script = "Draw=geometry;";
//    > {
//		SetVertexShader( CompileShader( vs_4_0, Fresnel_VS() ) );
//		
//		
//		SetRasterizerState(Rasterizer_Fresnel);
//		SetBlendState(Blend_Fresnel, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
//		SetDepthStencilState(Depth_Fresnel, 0);
//		
//		SetGeometryShader( NULL );
//		SetPixelShader( CompileShader( ps_4_0, Fresnel_PS() ) );
//
//	}
	pass p1 <
	string Script = "Draw=geometry;";
    > {
		SetVertexShader( CompileShader( vs_4_0, CookTorrance_VS() ) );
		
		
//		SetRasterizerState(Rasterizer_CookTorrance);
//		SetBlendState(Blend_CookTorrance, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
//		SetDepthStencilState(Depth_CookTorrance, 0);	
//		
//		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, CookTorrance_PS() ) );
		
	}
}
